@@ -1,12 +1,26 @@
+//! This module re-implements [`rustc_mir_dataflow::Engine`] for performance reasons.
+//!
+//! The Engine implementation in rustc optimizes for minimizing memory usage
+//! by only materializing results at the start of basic blocks, and recomputing
+//! the analysis when visiting results. However, this strategy involves a lot of
+//! creation and deletion of the [analysis domain][rustc_mir_dataflow::AnalysisDomain].
+//!
+//! The information flow analysis has a large domain of size `O(|places| * |locations|)`.
+//! Profiling showed that a significant portion of analysis time was just the engine
+//! allocating / cloning / dropping the domain, not doing computation. Therefore this
+//! engine improves performance but increases memory usage by up-front materializing
+//! the domain at every [`Location`].
+
 use std::rc::Rc;
 
+use either::Either;
 use rustc_data_structures::{graph::WithSuccessors, work_queue::WorkQueue};
 use rustc_index::vec::IndexVec;
 use rustc_middle::{
   mir::{traversal, Body, Location},
   ty::TyCtxt,
 };
-use rustc_mir_dataflow::*;
+use rustc_mir_dataflow::{Analysis, Direction, JoinSemiLattice, ResultsVisitor};
 
 use crate::indexed::{
   impls::{LocationDomain, LocationIndex},
@@ -67,6 +81,8 @@ pub fn iterate_to_fixpoint<'tcx, A: Analysis<'tcx>>(
 ) -> AnalysisResults<'tcx, A> {
   let bottom_value = analysis.bottom_value(body);
 
+  // `state` materializes the analysis domain for *every* location, which is the crux
+  // of this implementation strategy.
   let num_locs = location_domain.num_real_locations();
   let mut state = IndexVec::from_elem_n(bottom_value, num_locs);
 
@@ -85,31 +101,25 @@ pub fn iterate_to_fixpoint<'tcx, A: Analysis<'tcx>>(
     }
   }
 
-  let blocks = body.basic_blocks();
   while let Some(loc_index) = dirty_queue.pop() {
     let location = *location_domain.value(loc_index);
-    let data = &blocks[location.block];
-    let is_terminator = location.statement_index == data.statements.len();
-
-    if is_terminator {
-      let terminator = data.terminator();
-      analysis.apply_terminator_effect(&mut state[loc_index], terminator, location);
-    } else {
-      let statement = &data.statements[location.statement_index];
-      analysis.apply_statement_effect(&mut state[loc_index], statement, location);
-    }
-
-    let next_locs = if is_terminator {
-      body
-        .successors(location.block)
-        .map(|block| Location {
-          block,
-          statement_index: 0,
-        })
-        .collect::<Vec<_>>()
-    } else {
-      vec![location.successor_within_block()]
+    let next_locs = match body.stmt_at(location) {
+      Either::Left(statement) => {
+        analysis.apply_statement_effect(&mut state[loc_index], statement, location);
+        vec![location.successor_within_block()]
+      }
+      Either::Right(terminator) => {
+        analysis.apply_terminator_effect(&mut state[loc_index], terminator, location);
+        body
+          .successors(location.block)
+          .map(|block| Location {
+            block,
+            statement_index: 0,
+          })
+          .collect::<Vec<_>>()
+      }
     };
+
     for next_loc in next_locs {
       let next_loc_index = location_domain.index(&next_loc);
       let (cur_state, next_state) = state.pick2_mut(loc_index, next_loc_index);
